@@ -5,9 +5,9 @@
 #include "Common/Histogram.h"
 #include "Common/IOUtil.h"
 #include "Common/Sequence.h" // for isACGT
+#include "Common/Uncompress.h"
 #include "DataLayer/FastaReader.h"
 #include "DataLayer/Options.h"
-#include "Uncompress.h"
 #include <algorithm>
 #include <getopt.h>
 #include <iostream>
@@ -108,7 +108,7 @@ static void printContiguityStatistics(const char* path)
 		cout << "||"
 			<< "n" << sep
 			<< "n:" << opt::minLength << sep
-			<< "n:N50" << sep;
+			<< "L50" << sep;
 		if (opt::expSize > 0)
 			cout << "n:NG50" << sep
 				<< "NG50" << sep;
@@ -125,7 +125,7 @@ static void printContiguityStatistics(const char* path)
 		const char* sep = "\t|";
 		cout << "n" << sep
 			<< "n:" << opt::minLength << sep
-			<< "n:N50" << sep;
+			<< "L50" << sep;
 		if (opt::expSize > 0)
 			cout << "n:NG50" << sep
 				<< "NG50" << sep;
