@@ -1,7 +1,7 @@
 #ifndef DIRECTEDGRAPH_H
 #define DIRECTEDGRAPH_H 1
 
-#include "ContigNode.h"
+#include "Common/ContigNode.h"
 #include "Graph/Properties.h"
 #include <algorithm>
 #include <cassert>
@@ -594,6 +594,13 @@ num_vertices(const DirectedGraph<VP, EP>& g)
 	return g.num_vertices();
 }
 
+template <typename VP, typename EP>
+typename DirectedGraph<VP, EP>::vertex_descriptor
+vertex(typename DirectedGraph<VP, EP>::vertices_size_type ui, const DirectedGraph<VP, EP>& g)
+{
+	return g.vertex(ui);
+}
+
 template <typename VP, typename EP>
 std::pair<typename DirectedGraph<VP, EP>::vertex_iterator,
 	typename DirectedGraph<VP, EP>::vertex_iterator>
