@@ -31,18 +31,7 @@
 # include "Bloom/ConcurrentBloomFilter.h"
 #endif
 
-#undef USESEQAN
-
-#if USESEQAN
-#include <seqan/align.h>
-#include <seqan/sequence.h>
-#include <seqan/align_split.h>
-#endif
-
 using namespace std;
-#if USESEQAN
-using namespace seqan;
-#endif
 
 #define PROGRAM "konnector"
 
@@ -63,11 +52,27 @@ static const char USAGE_MESSAGE[] =
 "  -j, --threads=N            use N parallel threads [1]\n"
 "  -k, --kmer=N               the size of a k-mer\n"
 "  -b, --bloom-size=N         size of bloom filter [500M]\n"
+"  -c, --min-coverage=N       kmer coverage threshold for error correction [2].\n"
+"                             This option specifies the number of levels in the\n"
+"                             cascading Bloom filter; it has no effect if the Bloom\n"
+"                             filter is loaded from an external file.\n"
 "  -B, --max-branches=N       max branches in de Bruijn graph traversal;\n"
 "                             use 'nolimit' for no limit [350]\n"
 "  -d, --dot-file=FILE        write graph traversals to a DOT file\n"
+"  -D, --dup-bloom-size=N     use an additional Bloom filter to avoid\n"
+"                             assembling the same region of the genome\n"
+"                             multiple times. This option is highly\n"
+"                             recommended when the -E (--extend) option\n"
+"                             and has no effect otherwise. As a rule of\n"
+"                             thumb, the Bloom filter size should be\n"
+"                             about twice the target genome size [disabled]\n"
 "  -e, --fix-errors           find and fix single-base errors when reads\n"
 "                             have no kmers in bloom filter [disabled]\n"
+"  -E, --extend               in addition to finding a connecting path,\n"
+"                             extend the reads outwards to the next\n"
+"                             dead end or branching point in the de Brujin\n"
+"                             graph. If the reads were not successfully\n"
+"                             connected, extend them inwards as well.\n"
 "  -f, --min-frag=N           min fragment size in base pairs [0]\n"
 "  -F, --max-frag=N           max fragment size in base pairs [1000]\n"
 "  -i, --input-bloom=FILE     load bloom filter from FILE\n"
@@ -79,10 +84,6 @@ static const char USAGE_MESSAGE[] =
 "      --trim-masked          trim masked bases from the ends of reads\n"
 "      --no-trim-masked       do not trim masked bases from the ends\n"
 "                             of reads [default]\n"
-"  -l, --long-search          start path search as close as possible\n"
-"                             to the beginnings of reads. Takes more time\n"
-"                             but improves results when bloom filter false\n"
-"                             positive rate is high [disabled]\n"
 "  -m, --read-mismatches=N    max mismatches between paths and reads; use\n"
 "                             'nolimit' for no limit [nolimit]\n"
 "  -M, --max-mismatches=N     max mismatches between all alternate paths;\n"
@@ -110,6 +111,19 @@ static const char USAGE_MESSAGE[] =
 "Report bugs to <" PACKAGE_BUGREPORT ">.\n";
 
 const unsigned g_progressStep = 1000;
+/*
+ * ignore branches less than this length
+ *(false positive branches)
+ */
+const unsigned g_trimLen = 3;
+
+/*
+ * Bloom filter use to keep track of portions
+ * of genome that have already been assembled.
+ * This Bloom filter is only used when the
+ * -E (--extend) option is in effect.
+ */
+BloomFilter g_dupBloom;
 
 namespace opt {
 
@@ -119,28 +133,38 @@ namespace opt {
 	/** The size of the bloom filter in bytes. */
 	size_t bloomSize = 500 * 1024 * 1024;
 
+	/** The maximum count value of the Bloom filter. */
+	unsigned minCoverage = 2;
+
 	/** Input read files are interleaved? */
 	bool interleaved = false;
 
-	/**
-	 * Choose start/goal kmers for path search as close as
-	 * possible to beginning (5' end) of reads. Improves
-	 * results when bloom filter FPR is high.
-	 */
-	bool longSearch = false;
-
 	/** Max active branches during de Bruijn graph traversal */
 	unsigned maxBranches = 350;
 
 	/** multi-graph DOT file containing graph traversals */
 	static string dotPath;
 
+	/**
+	 * Dup Bloom filter size.
+	 * The dup filter is used to avoid assembling duplicate
+	 * sequences when the -E (--extend) option is in effect.
+	 */
+	size_t dupBloomSize = 0;
+
 	/**
 	 * Find and fix single base errors when a read has no
 	 * kmers in the bloom filter.
 	 */
 	bool fixErrors = false;
 
+	/**
+	 * Extend reads outwards until the next dead or branching
+	 * point in the de Bruijn graph.  If a read pair is not
+	 * successfully connected, extend them inwards as well.
+	 */
+	bool extend = false;
+
 	/** The size of a k-mer. */
 	unsigned k;
 
@@ -195,22 +219,28 @@ static struct {
 	size_t readPairsProcessed;
 	size_t readPairsMerged;
 	size_t skipped;
+	/* counts below are used only when -E is enabled */
+	size_t mergedAndExtended;
+	size_t mergedAndSkipped;
+	size_t singleEndCorrected;
 } g_count;
 
-static const char shortopts[] = "b:B:d:ef:F:i:Ij:k:lm:M:no:P:q:r:s:t:v";
+static const char shortopts[] = "b:B:c:d:D:eEf:F:i:Ij:k:lm:M:no:P:q:r:s:t:v";
 
 enum { OPT_HELP = 1, OPT_VERSION };
 
 static const struct option longopts[] = {
 	{ "bloom-size",       required_argument, NULL, 'b' },
+	{ "min-coverage",     required_argument, NULL, 'c' },
 	{ "max-branches",     required_argument, NULL, 'B' },
 	{ "dot-file",         required_argument, NULL, 'd' },
+	{ "dup-bloom-size",   required_argument, NULL, 'D' },
 	{ "fix-errors",       no_argument, NULL, 'e' },
+	{ "extend",           no_argument, NULL, 'E' },
 	{ "min-frag",         required_argument, NULL, 'f' },
 	{ "max-frag",         required_argument, NULL, 'F' },
 	{ "input-bloom",      required_argument, NULL, 'i' },
 	{ "interleaved",      no_argument, NULL, 'I' },
-	{ "long-search",      no_argument, NULL, 'l' },
 	{ "threads",          required_argument, NULL, 'j' },
 	{ "kmer",             required_argument, NULL, 'k' },
 	{ "chastity",         no_argument, &opt::chastityFilter, 1 },
@@ -236,178 +266,422 @@ static const struct option longopts[] = {
 	{ NULL, 0, NULL, 0 }
 };
 
-#if USESEQAN
-const string r1 =
-"AGAATCAACCAACCGTTCAATGATATAATCAAGAGCGATATTGTAATCTTTGTTTCT";
-const string r2 =
-"CGACGTCCACCAATTCGTCCCTGTGCACGAGCAGTTTCCAGTCCAGCTTTTGTTCGT";
-const string ins =
-"AGAATCAACCAACCGTTCAATGATATAATCAAGAGCGATATTGTAATCTTTGTTTCTGTCACCCGGCCCCCACGACTCAAGGATTAGACCATAAACACCATCCTCTTCACCTATCGAACACTCAGCTTTCAGTTCAATTCCATTATTATCAAAAACATGCATAATATTAATCTTTAATCAATTTTTCACGACAATACTACTTTTATTGATAAAATTGCAACAAGTTGCTGTTGTTTTACTTTCTTTTGTACACAAAGTGTCTTTAACTTTATTTATCCCCTGCAGGAAACCTCTTATACAAAGTTGACACACCAACATCATAGATAATCGCCACCTTCTGGCGAGGAGTTCCTGCTGCAATTAATCGTCCAGCTTGTGCCCATTGTTCTGGTGTAAGTTTGGGACGACGTCCACCAATTCGTCCCTGTGCACGAGCAGTTTCCAGTCCAGCTTTTGTTCGT";
-
-static void seqanTests()
+/**
+ * Return true if the Bloom filter contains all of the
+ * kmers in the given sequence.
+ */
+static bool bloomContainsSeq(const BloomFilter& bloom, const Sequence& seq)
 {
-	typedef String<Dna> DS;
-	typedef Align<DS> Alignment;
-
-    //DS seq1 = "TTGT";
-    //DS seq2 = "TTAGT";
-	DS ref = ins;
-	DS seq1 = r1;
-	DS seq2 = r2;
-
-    Alignment align1;
-	resize(rows(align1), 2);
-	assignSource(row(align1, 0), ref);
-	assignSource(row(align1, 1), seq1);
-    Alignment align2;
-	resize(rows(align2), 2);
-	assignSource(row(align2, 0), ref);
-	assignSource(row(align2, 1), seq2);
-
-	Score<int> scoring(2, -2, -50, -100);
-
-	cout << splitAlignment(align1, align2, scoring) << endl;
-	cout << align1 << endl;
-	cout << align2 << endl;
-
-	cout << localAlignment(align1, scoring) << endl;
-	cout << align1 << endl;
-
-	cout << localAlignment(align2, scoring) << endl;
-	cout << align2 << endl;
+	if (containsAmbiguityCodes(seq)) {
+		Sequence seqCopy = seq;
+		flattenAmbiguityCodes(seqCopy, false);
+		for (KmerIterator it(seqCopy, opt::k); it != KmerIterator::end();
+			++it) {
+			if (!bloom[*it])
+				return false;
+		}
+		return true;
+	}
+	for (KmerIterator it(seq, opt::k); it != KmerIterator::end(); ++it) {
+		if (!bloom[*it])
+			return false;
+	}
+	return true;
+}
+
+/**
+ * Load the kmers of a given sequence into a Bloom filter.
+ */
+static inline void loadSeq(BloomFilter& bloom, unsigned k, const Sequence& seq)
+{
+	if (containsAmbiguityCodes(seq)) {
+		Sequence seqCopy = seq;
+		Sequence rc = reverseComplement(seqCopy);
+		flattenAmbiguityCodes(seqCopy, false);
+		flattenAmbiguityCodes(rc, false);
+		Bloom::loadSeq(bloom, k, seqCopy);
+		Bloom::loadSeq(bloom, k, rc);
+	} else {
+		Bloom::loadSeq(bloom, k, seq);
+	}
+}
+
+/**
+ * Extend a read/pseudoread both left and right until
+ * we hit the next dead end or branching point in the
+ * de Bruijn graph.
+ *
+ * @param seq sequence to be extended
+ * @param k kmer size
+ * @param g de Bruijn graph
+ * return true if the read was extended in either
+ * (or both) directions, false otherwise
+ */
+template <typename Graph>
+static bool extendRead(Sequence& seq, unsigned k, const Graph& g)
+{
+	ExtendSeqResult result;
+	bool extended = false;
+
+	/*
+	 * offset start pos to reduce chance of hitting
+	 * a dead end on a false positive kmer
+	 */
+	const unsigned runLengthHint = 3;
+	unsigned startPos = getStartKmerPos(seq, k, FORWARD, g,
+		runLengthHint);
+	if (startPos != NO_MATCH) {
+		assert(startPos <= seq.length() - k);
+		result = extendSeq(seq, FORWARD, startPos, k, g,
+				NO_LIMIT, g_trimLen, opt::mask);
+		if (result == ES_EXTENDED_TO_DEAD_END ||
+				result == ES_EXTENDED_TO_BRANCHING_POINT ||
+				result == ES_EXTENDED_TO_CYCLE) {
+			extended = true;
+		}
+	}
+
+	startPos = getStartKmerPos(seq, k, REVERSE, g, runLengthHint);
+	if (startPos != NO_MATCH) {
+		assert(startPos <= seq.length() - k);
+		result = extendSeq(seq, REVERSE, startPos, k, g,
+				NO_LIMIT, g_trimLen, opt::mask);
+		if (result == ES_EXTENDED_TO_DEAD_END ||
+				result == ES_EXTENDED_TO_BRANCHING_POINT ||
+				result == ES_EXTENDED_TO_CYCLE) {
+			extended = true;
+		}
+	}
+
+	return extended;
+}
+
+enum ExtendResult { ER_NOT_EXTENDED, ER_REDUNDANT, ER_EXTENDED };
+
+/**
+ * Attempt to extend a merged read (a.k.a. pseudoread)
+ * outward to the next branching point or dead end in
+ * the de Bruijn graph.
+ *
+ * @param seq pseudoread to be extended
+ * @param k kmer size
+ * @param g de Bruijn graph in which to perform extension
+ * @return ExtendResult (ER_NOT_EXTENDED, ER_EXTENDED,
+ * ER_REDUNDANT)
+ */
+template <typename Graph>
+static inline ExtendResult
+extendReadIfNonRedundant(Sequence& seq, unsigned k, const Graph& g)
+{
+	bool redundant = false;
+	if (opt::dupBloomSize > 0) {
+		/*
+		 * Check to see if the current pseudoread
+		 * is contained in a region of the genome
+		 * that has already been assembled.
+		 */
+#pragma omp critical(dupBloom)
+		redundant = bloomContainsSeq(g_dupBloom, seq);
+		if (redundant)
+			return ER_REDUNDANT;
+	}
+	Sequence origSeq = seq;
+	bool extended = extendRead(seq, k, g);
+	if (opt::dupBloomSize > 0) {
+		/*
+		 * mark the extended read as an assembled
+		 * region of the genome.
+		 */
+#pragma omp critical(dupBloom)
+		{
+			/* must check again to avoid race conditions */
+			if (!bloomContainsSeq(g_dupBloom, origSeq))
+				loadSeq(g_dupBloom, opt::k, seq);
+			else
+				redundant = true;
+		}
+		if (redundant)
+			return ER_REDUNDANT;
+	}
+	assert(!redundant);
+	if (extended)
+		return ER_EXTENDED;
+	else
+		return ER_NOT_EXTENDED;
+}
+
+/**
+ * Print progress stats about reads merged/extended so far.
+ */
+static inline void printProgressMessage()
+{
+	cerr << "Merged " << g_count.uniquePath + g_count.multiplePaths << " of "
+		<< g_count.readPairsProcessed << " read pairs";
+
+	if (opt::extend) {
+		cerr << ", corrected/extended " << g_count.singleEndCorrected << " of "
+			<< (g_count.readPairsProcessed - g_count.uniquePath -
+				g_count.multiplePaths) * 2
+		<< " unmerged reads";
+	}
+
+	cerr << " (no start/goal kmer: " << g_count.noStartOrGoalKmer << ", "
+		<< "no path: " << g_count.noPath << ", "
+		<< "too many paths: " << g_count.tooManyPaths << ", "
+		<< "too many branches: " << g_count.tooManyBranches << ", "
+		<< "too many path/path mismatches: " << g_count.tooManyMismatches << ", "
+		<< "too many path/read mismatches: " << g_count.tooManyReadMismatches << ", "
+		<< "contains cycle: " << g_count.containsCycle << ", "
+		<< "skipped: " << g_count.skipped
+		<< ")\n";
+}
+
+
+/**
+ * For a successfully merged read pair, get the sequence
+ * representing the connecting path between the two reads.
+ */
+template <typename Bloom>
+static inline string getConnectingSeq(ConnectPairsResult& result,
+	unsigned k, const Bloom& bloom)
+{
+	assert(result.pathResult == FOUND_PATH);
+	(void)bloom;
+
+	vector<FastaRecord>& paths = result.mergedSeqs;
+	assert(paths.size() > 0);
+
+	Sequence& seq = (paths.size() == 1) ?
+		paths.front().seq : result.consensusSeq.seq;
+
+	/*
+	 * initialize sequence to the chars between the
+	 * start and goal kmers of the path search.
+	 */
+	int startPos = result.startKmerPos;
+	int endPos = seq.length() - result.goalKmerPos - k;
+	assert(startPos >= 0 && startPos <=
+		(int)(seq.length() - k + 1));
+
+	return seq.substr(startPos, endPos - startPos + k);
 }
-#endif
 
 /** Connect a read pair. */
-template <typename Graph>
+template <typename Graph, typename Bloom>
 static void connectPair(const Graph& g,
-	const FastqRecord& read1,
-	const FastqRecord& read2,
+	const Bloom& bloom,
+	FastqRecord& read1,
+	FastqRecord& read2,
 	const ConnectPairsParams& params,
 	ofstream& mergedStream,
 	ofstream& read1Stream,
 	ofstream& read2Stream,
 	ofstream& traceStream)
 {
-	bool skip = false;
-
+	/*
+	 * Implements the -r option, which is used to only
+	 * process a subset of the input read pairs.
+	 */
 	if (!opt::readName.empty() &&
 		read1.id.find(opt::readName) == string::npos) {
 #pragma omp atomic
 		++g_count.skipped;
-		skip = true;
+		return;
 	}
 
-	if (!skip) {
+	ConnectPairsResult result =
+		connectPairs(opt::k, read1, read2, g, params);
 
-		ConnectPairsResult result =
-			connectPairs(opt::k, read1, read2, g, params);
+	vector<FastaRecord>& paths = result.mergedSeqs;
+	bool mergedSeqRedundant = false;
+	bool read1Corrected = false;
+	bool read1Redundant = false;
+	bool read2Corrected = false;
+	bool read2Redundant = false;
 
-		vector<FastaRecord>& paths = result.mergedSeqs;
+	/*
+	 * extend reads inwards or outwards up to the
+	 * next dead end or branching point in the de
+	 * Brujin graph
+	 */
+	if (opt::extend) {
+		ExtendResult extendResult;
+		if (result.pathResult == FOUND_PATH
+			&& result.pathMismatches <= params.maxPathMismatches
+			&& result.readMismatches <= params.maxReadMismatches) {
+			assert(paths.size() > 0);
+			Sequence& seq = (paths.size() == 1) ?
+				paths.front().seq : result.consensusSeq.seq;
+			seq = getConnectingSeq(result, opt::k, bloom);
+			extendResult = extendReadIfNonRedundant(
+				seq, opt::k, g);
+			if (extendResult == ER_REDUNDANT) {
+#pragma omp atomic
+				g_count.mergedAndSkipped++;
+				mergedSeqRedundant = true;
+			} else if (extendResult == ER_EXTENDED) {
+#pragma omp atomic
+				g_count.mergedAndExtended++;
+			}
+		} else {
 
-		if (!opt::tracefilePath.empty())
-#pragma omp critical(tracefile)
-		{
-			traceStream << result;
-			assert_good(traceStream, opt::tracefilePath);
+			/*
+			 * read pair could not be merged, so try
+			 * to extend each read individually (in
+			 * both directions).
+			 */
+
+//std::cerr << "correcting " << read1.id << " (read 1)" << std::endl;
+			read1Corrected = correctAndExtendSeq(read1.seq,
+				opt::k, g, read1.seq.length(), g_trimLen,
+				opt::mask);
+
+			if (read1Corrected) {
+#pragma omp atomic
+				g_count.singleEndCorrected++;
+				extendResult = extendReadIfNonRedundant(read1.seq,
+					opt::k, g);
+				if (extendResult == ER_REDUNDANT)
+					read1Redundant = true;
+			}
+
+//std::cerr << "correcting " << read2.id << " (read 2)" << std::endl;
+			read2Corrected = correctAndExtendSeq(read2.seq,
+				opt::k, g, read2.seq.length(), g_trimLen,
+				opt::mask);
+
+			if (read2Corrected) {
+#pragma omp atomic
+				g_count.singleEndCorrected++;
+				extendResult = extendReadIfNonRedundant(read2.seq,
+					opt::k, g);
+				if (extendResult == ER_REDUNDANT)
+					read2Redundant = true;
+			}
 		}
+	}
 
-		switch (result.pathResult) {
+	if (!opt::tracefilePath.empty())
+#pragma omp critical(tracefile)
+	{
+		traceStream << result;
+		assert_good(traceStream, opt::tracefilePath);
+	}
 
-			case NO_PATH:
-				assert(paths.empty());
-				if (result.foundStartKmer && result.foundGoalKmer)
-#pragma omp atomic
-					++g_count.noPath;
-				else {
-#pragma omp atomic
-					++g_count.noStartOrGoalKmer;
-				}
-				break;
+	switch (result.pathResult) {
 
-			case FOUND_PATH:
-				assert(!paths.empty());
-				if (result.pathMismatches > params.maxPathMismatches ||
-						result.readMismatches > params.maxReadMismatches) {
-					if (result.pathMismatches > params.maxPathMismatches)
+		case NO_PATH:
+			assert(paths.empty());
+			if (result.foundStartKmer && result.foundGoalKmer)
 #pragma omp atomic
-						++g_count.tooManyMismatches;
-					else
-						++g_count.tooManyReadMismatches;
-#pragma omp critical(read1Stream)
+				++g_count.noPath;
+			else {
+#pragma omp atomic
+				++g_count.noStartOrGoalKmer;
+			}
+			break;
+
+		case FOUND_PATH:
+			assert(!paths.empty());
+			if (result.pathMismatches > params.maxPathMismatches ||
+					result.readMismatches > params.maxReadMismatches) {
+				if (result.pathMismatches > params.maxPathMismatches)
+#pragma omp atomic
+					++g_count.tooManyMismatches;
+				else
+					++g_count.tooManyReadMismatches;
+				if (opt::extend) {
+					if (read1Corrected || read2Corrected)
+#pragma omp critical(mergedStream)
+					{
+						if (read1Corrected && !read1Redundant)
+							mergedStream << (FastaRecord)read1;
+						if (read2Corrected && !read2Redundant)
+							mergedStream << (FastaRecord)read2;
+					}
+					if (!read1Corrected || !read2Corrected)
+#pragma omp critical(readStream)
+					{
+						if (!read1Corrected)
+							read1Stream << (FastaRecord)read1;
+						if (!read2Corrected)
+							read1Stream << (FastaRecord)read2;
+					}
+				} else
+#pragma omp critical(readStream)
+				{
 					read1Stream << read1;
-#pragma omp critical(read2Stream)
 					read2Stream << read2;
 				}
-				else if (paths.size() > 1) {
+			}
+			else if (paths.size() > 1) {
 #pragma omp atomic
-					++g_count.multiplePaths;
+				++g_count.multiplePaths;
+				if (!mergedSeqRedundant)
 #pragma omp critical(mergedStream)
 					mergedStream << result.consensusSeq;
-				}
-				else {
+			}
+			else {
 #pragma omp atomic
-					++g_count.uniquePath;
+				++g_count.uniquePath;
+				if (!mergedSeqRedundant)
 #pragma omp critical(mergedStream)
 					mergedStream << paths.front();
-				}
-				break;
+			}
+			break;
 
-			case TOO_MANY_PATHS:
+		case TOO_MANY_PATHS:
 #pragma omp atomic
-				++g_count.tooManyPaths;
-				break;
+			++g_count.tooManyPaths;
+			break;
 
-			case TOO_MANY_BRANCHES:
+		case TOO_MANY_BRANCHES:
 #pragma omp atomic
-				++g_count.tooManyBranches;
-				break;
+			++g_count.tooManyBranches;
+			break;
 
-			case PATH_CONTAINS_CYCLE:
+		case PATH_CONTAINS_CYCLE:
 #pragma omp atomic
-				++g_count.containsCycle;
-				break;
+			++g_count.containsCycle;
+			break;
 
-			case EXCEEDED_MEM_LIMIT:
+		case EXCEEDED_MEM_LIMIT:
 #pragma omp atomic
-				++g_count.exceededMemLimit;
-				break;
-		}
-
-		if (result.pathResult != FOUND_PATH) {
-#pragma omp critical(read1Stream)
-			read1Stream << read1;
-#pragma omp critical(read2Stream)
-			read2Stream << read2;
-		}
+			++g_count.exceededMemLimit;
+			break;
 	}
 
-#pragma omp critical(cerr)
-	{
-		g_count.readPairsProcessed++;
-		if (opt::verbose >= 2)
-		{
-			if(g_count.readPairsProcessed % g_progressStep == 0) {
-				cerr << "Merged " << g_count.uniquePath + g_count.multiplePaths << " of "
-					<< g_count.readPairsProcessed << " read pairs "
-					<< "(no start/goal kmer: " << g_count.noStartOrGoalKmer << ", "
-					<< "no path: " << g_count.noPath << ", "
-					<< "too many paths: " << g_count.tooManyPaths << ", "
-					<< "too many branches: " << g_count.tooManyBranches << ", "
-					<< "too many path/path mismatches: " << g_count.tooManyMismatches << ", "
-					<< "too many path/read mismatches: " << g_count.tooManyReadMismatches << ", "
-					<< "contains cycle: " << g_count.containsCycle << ", "
-					<< "exceeded mem limit: " << g_count.exceededMemLimit << ", "
-					<< "skipped: " << g_count.skipped
-					<< ")\n";
+	if (result.pathResult != FOUND_PATH) {
+		if (opt::extend) {
+			if (read1Corrected || read2Corrected)
+#pragma omp critical(mergedStream)
+			{
+				if (read1Corrected && !read1Redundant)
+					mergedStream << (FastaRecord)read1;
+				if (read2Corrected && !read2Redundant)
+					mergedStream << (FastaRecord)read2;
 			}
+			if (!read1Corrected || !read2Corrected)
+#pragma omp critical(readStream)
+			{
+				if (!read1Corrected)
+					read1Stream << (FastaRecord)read1;
+				if (!read2Corrected)
+					read1Stream << (FastaRecord)read2;
+			}
+		} else
+#pragma omp critical(readStream)
+		{
+			read1Stream << read1;
+			read2Stream << read2;
 		}
 	}
 }
 
 /** Connect read pairs. */
-template <typename Graph, typename FastaStream>
+template <typename Graph, typename FastaStream, typename Bloom>
 static void connectPairs(const Graph& g,
+	const Bloom& bloom,
 	FastaStream& in,
 	const ConnectPairsParams& params,
 	ofstream& mergedStream,
@@ -420,11 +694,20 @@ static void connectPairs(const Graph& g,
 		bool good;
 #pragma omp critical(in)
 		good = in >> a >> b;
-		if (good)
-			connectPair(g, a, b, params, mergedStream, read1Stream,
+		if (good) {
+			connectPair(g, bloom, a, b, params, mergedStream, read1Stream,
 				read2Stream, traceStream);
-		else
+#pragma omp atomic
+			g_count.readPairsProcessed++;
+			if (opt::verbose >= 2)
+#pragma omp critical(cerr)
+			{
+				if(g_count.readPairsProcessed % g_progressStep == 0)
+					printProgressMessage();
+			}
+		} else {
 			break;
+		}
 	}
 }
 
@@ -436,7 +719,7 @@ static inline void setMaxOption(unsigned& arg, istream& in)
 {
 	string str;
 	getline(in, str);
-	if (in.good() && str == "nolimit") {
+	if (!in.fail() && str.compare("nolimit")==0) {
 		arg = NO_LIMIT;
 	} else {
 		istringstream ss(str);
@@ -453,6 +736,7 @@ static inline void setMaxOption(unsigned& arg, istream& in)
 int main(int argc, char** argv)
 {
 	bool die = false;
+	bool minCovOptUsed = false;
 
 	for (int c; (c = getopt_long(argc, argv,
 					shortopts, longopts, NULL)) != -1;) {
@@ -462,12 +746,20 @@ int main(int argc, char** argv)
 			die = true; break;
 		  case 'b':
 			opt::bloomSize = SIToBytes(arg); break;
+		  case 'c':
+			arg >> opt::minCoverage;
+			minCovOptUsed = true;
+			break;
 		  case 'B':
 			setMaxOption(opt::maxBranches, arg); break;
 		  case 'd':
 			arg >> opt::dotPath; break;
+		  case 'D':
+			opt::dupBloomSize = SIToBytes(arg); break;
 		  case 'e':
 			opt::fixErrors = true; break;
+		  case 'E':
+			opt::extend = true; break;
 		  case 'f':
 			arg >> opt::minFrag; break;
 		  case 'F':
@@ -480,8 +772,6 @@ int main(int argc, char** argv)
 			arg >> opt::threads; break;
 		  case 'k':
 			arg >> opt::k; break;
-		  case 'l':
-			opt::longSearch = true; break;
 		  case 'm':
 			setMaxOption(opt::maxReadMismatches, arg); break;
 		  case 'n':
@@ -541,6 +831,12 @@ int main(int argc, char** argv)
 		exit(EXIT_FAILURE);
 	}
 
+	if (!opt::inputBloomPath.empty() && minCovOptUsed) {
+		cerr << PROGRAM ": warning: -c option has no effect when "
+			" using a pre-built Bloom filter (-i option)\n";
+	}
+
+
 #if _OPENMP
 	if (opt::threads > 0)
 		omp_set_num_threads(opt::threads);
@@ -554,7 +850,11 @@ int main(int argc, char** argv)
 
 	assert(opt::bloomSize > 0);
 
-	BloomFilter bloom;
+	if (opt::dupBloomSize > 0)
+		g_dupBloom.resize(opt::dupBloomSize * 8);
+
+	BloomFilter* bloom;
+	CascadingBloomFilter* cascadingBloom = NULL;
 
 	if (!opt::inputBloomPath.empty()) {
 
@@ -562,34 +862,40 @@ int main(int argc, char** argv)
 			std::cerr << "Loading bloom filter from `"
 				<< opt::inputBloomPath << "'...\n";
 
+		bloom = new BloomFilter();
+
 		const char* inputPath = opt::inputBloomPath.c_str();
 		ifstream inputBloom(inputPath, ios_base::in | ios_base::binary);
 		assert_good(inputBloom, inputPath);
-		inputBloom >> bloom;
+		inputBloom >> *bloom;
 		assert_good(inputBloom, inputPath);
 		inputBloom.close();
 
 	} else {
 
-		// Specify bloom filter size in bits. Divide by two
-		// because counting bloom filter requires twice as
-		// much space.
-		size_t bits = opt::bloomSize * 8 / 2;
-		CascadingBloomFilter tempBloom(bits);
+		if (opt::verbose)
+			std::cerr << "Using a minimum kmer coverage threshold of "
+				<< opt::minCoverage << "\n";
+
+		// Specify bloom filter size in bits and divide by number
+		// of levels in cascading Bloom filter.
+
+		size_t bits = opt::bloomSize * 8 / opt::minCoverage;
+		cascadingBloom = new CascadingBloomFilter(bits, opt::minCoverage);
 #ifdef _OPENMP
-		ConcurrentBloomFilter<CascadingBloomFilter> cbf(tempBloom, 1000);
+		ConcurrentBloomFilter<CascadingBloomFilter> cbf(*cascadingBloom, 1000);
 		for (int i = optind; i < argc; i++)
 			Bloom::loadFile(cbf, opt::k, string(argv[i]), opt::verbose);
 #else
 		for (int i = optind; i < argc; i++)
-			Bloom::loadFile(tempBloom, opt::k, string(argv[i]), opt::verbose);
+			Bloom::loadFile(*cascadingBloom, opt::k, string(argv[i]), opt::verbose);
 #endif
-		bloom = tempBloom.getBloomFilter(tempBloom.MAX_COUNT-1);
+		bloom = &cascadingBloom->getBloomFilter(opt::minCoverage - 1);
 	}
 
 	if (opt::verbose)
 		cerr << "Bloom filter FPR: " << setprecision(3)
-			<< 100 * bloom.FPR() << "%\n";
+			<< 100 * bloom->FPR() << "%\n";
 
 	ofstream dotStream;
 	if (!opt::dotPath.empty()) {
@@ -611,20 +917,40 @@ int main(int argc, char** argv)
 		assert_good(traceStream, opt::tracefilePath);
 	}
 
-	DBGBloom<BloomFilter> g(bloom);
+	DBGBloom<BloomFilter> g(*bloom);
+
+	/*
+	 * read pairs that were successfully connected
+	 * (and possibly extended outwards)
+	 */
 
 	string mergedOutputPath(opt::outputPrefix);
-	mergedOutputPath.append("_merged.fa");
+	if (opt::extend)
+		mergedOutputPath.append("_pseudoreads.fa");
+	else
+		mergedOutputPath.append("_merged.fa");
 	ofstream mergedStream(mergedOutputPath.c_str());
 	assert_good(mergedStream, mergedOutputPath);
 
+	/*
+	 * read pairs that were not successfully connected,
+	 * but may have been extended inwards and/or outwards
+	 * (if -E option was used)
+	 */
+
 	string read1OutputPath(opt::outputPrefix);
-	read1OutputPath.append("_reads_1.fq");
+	if (opt::extend)
+		read1OutputPath.append("_reads_1.fa");
+	else
+		read1OutputPath.append("_reads_1.fq");
 	ofstream read1Stream(read1OutputPath.c_str());
 	assert_good(read1Stream, read1OutputPath);
 
 	string read2OutputPath(opt::outputPrefix);
-	read2OutputPath.append("_reads_2.fq");
+	if (opt::extend)
+		read2OutputPath.append("_reads_2.fa");
+	else
+		read2OutputPath.append("_reads_2.fq");
 	ofstream read2Stream(read2OutputPath.c_str());
 	assert_good(read2Stream, read2OutputPath);
 
@@ -639,8 +965,8 @@ int main(int argc, char** argv)
 	params.maxBranches = opt::maxBranches;
 	params.maxPathMismatches = opt::maxMismatches;
 	params.maxReadMismatches = opt::maxReadMismatches;
+	params.kmerMatchesThreshold = 3;
 	params.fixErrors = opt::fixErrors;
-	params.longSearch = opt::longSearch;
 	params.maskBases = opt::mask;
 	params.memLimit = opt::searchMem;
 	params.dotPath = opt::dotPath;
@@ -649,13 +975,13 @@ int main(int argc, char** argv)
 	if (opt::interleaved) {
 		FastaConcat in(argv + optind, argv + argc,
 				FastaReader::FOLD_CASE);
-		connectPairs(g, in, params, mergedStream, read1Stream,
+		connectPairs(g, *bloom, in, params, mergedStream, read1Stream,
 				read2Stream, traceStream);
 		assert(in.eof());
 	} else {
 		FastaInterleave in(argv + optind, argv + argc,
 				FastaReader::FOLD_CASE);
-		connectPairs(g, in, params, mergedStream, read1Stream,
+		connectPairs(g, *bloom, in, params, mergedStream, read1Stream,
 				read2Stream, traceStream);
 		assert(in.eof());
 	}
@@ -705,18 +1031,27 @@ int main(int argc, char** argv)
 				<< " (" << setprecision(3) << (float)100
 					* g_count.containsCycle / g_count.readPairsProcessed
 				<< "%)\n"
-			"Exceeded mem limit: " << g_count.exceededMemLimit
-				<< " (" << setprecision(3) << (float)100
-					* g_count.exceededMemLimit / g_count.readPairsProcessed
-				<< "%)\n"
 			"Skipped: " << g_count.skipped
 				<< " (" << setprecision(3) << (float)100
 					* g_count.skipped / g_count.readPairsProcessed
-				<< "%)\n"
-			"Bloom filter FPR: " << setprecision(3) << 100 * bloom.FPR()
-				<< "%\n";
+				<< "%)\n";
+			if (opt::extend) {
+				cerr << "Unmerged reads corrected/extended: "
+					<< g_count.singleEndCorrected
+					<< " (" << setprecision(3) <<  (float)100
+					* g_count.singleEndCorrected / ((g_count.readPairsProcessed -
+					g_count.uniquePath - g_count.multiplePaths) * 2)
+					<< "%)\n";
+			}
+			std::cerr << "Bloom filter FPR: " << setprecision(3)
+				<< 100 * bloom->FPR() << "%\n";
 	}
 
+	if (!opt::inputBloomPath.empty())
+		delete bloom;
+	else
+		delete cascadingBloom;
+
 	assert_good(mergedStream, mergedOutputPath.c_str());
 	mergedStream.close();
 	assert_good(read1Stream, read1OutputPath.c_str());
